'use strict';

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _Const = require('../Const');

var _Const2 = _interopRequireDefault(_Const);

var legalComparators = ['=', '>', '>=', '<', '<=', '!='];

var NumberFilter = (function (_Component) {
  _inherits(NumberFilter, _Component);

  function NumberFilter(props) {
    _classCallCheck(this, NumberFilter);

    _Component.call(this, props);
    this.numberComparators = this.props.numberComparators || legalComparators;
    this.timeout = null;
    this.state = {
      isPlaceholderSelected: this.props.defaultValue === undefined || this.props.defaultValue.number === undefined || this.props.options && this.props.options.indexOf(this.props.defaultValue.number) === -1
    };
    this.onChangeNumber = this.onChangeNumber.bind(this);
    this.onChangeNumberSet = this.onChangeNumberSet.bind(this);
    this.onChangeComparator = this.onChangeComparator.bind(this);
  }

  NumberFilter.prototype.onChangeNumber = function onChangeNumber(event) {
    var _this = this;

    var comparator = this.refs.numberFilterComparator.value;
    if (comparator === '') {
      return;
    }
    if (this.timeout) {
      clearTimeout(this.timeout);
    }
    var filterValue = event.target.value;
    this.timeout = setTimeout(function () {
      _this.props.filterHandler({ number: filterValue, comparator: comparator }, _Const2['default'].FILTER_TYPE.NUMBER);
    }, this.props.delay);
  };

  NumberFilter.prototype.onChangeNumberSet = function onChangeNumberSet(event) {
    var comparator = this.refs.numberFilterComparator.value;
    var value = event.target.value;

    this.setState({ isPlaceholderSelected: value === '' });
    if (comparator === '') {
      return;
    }
    this.props.filterHandler({ number: value, comparator: comparator }, _Const2['default'].FILTER_TYPE.NUMBER);
  };

  NumberFilter.prototype.onChangeComparator = function onChangeComparator(event) {
    var value = this.refs.numberFilter.value;
    var comparator = event.target.value;
    if (value === '') {
      return;
    }
    this.props.filterHandler({ number: value, comparator: comparator }, _Const2['default'].FILTER_TYPE.NUMBER);
  };

  NumberFilter.prototype.getComparatorOptions = function getComparatorOptions() {
    var optionTags = [];
    optionTags.push(_react2['default'].createElement('option', { key: '-1' }));
    for (var i = 0; i < this.numberComparators.length; i++) {
      optionTags.push(_react2['default'].createElement(
        'option',
        { key: i, value: this.numberComparators[i] },
        this.numberComparators[i]
      ));
    }
    return optionTags;
  };

  NumberFilter.prototype.getNumberOptions = function getNumberOptions() {
    var optionTags = [];
    var options = this.props.options;

    optionTags.push(_react2['default'].createElement(
      'option',
      { key: '-1', value: '' },
      this.props.placeholder || 'Select ' + this.props.columnName + '...'
    ));
    for (var i = 0; i < options.length; i++) {
      optionTags.push(_react2['default'].createElement(
        'option',
        { key: i, value: options[i] },
        options[i]
      ));
    }
    return optionTags;
  };

  NumberFilter.prototype.componentDidMount = function componentDidMount() {
    var comparator = this.refs.numberFilterComparator.value;
    var number = this.refs.numberFilter.value;
    if (comparator && number) {
      this.props.filterHandler({ number: number, comparator: comparator }, _Const2['default'].FILTER_TYPE.NUMBER);
    }
  };

  NumberFilter.prototype.componentWillUnmount = function componentWillUnmount() {
    clearTimeout(this.timeout);
  };

  NumberFilter.prototype.render = function render() {
    var selectClass = _classnames2['default']('select-filter', 'number-filter-input', 'form-control', { 'placeholder-selected': this.state.isPlaceholderSelected });

    return _react2['default'].createElement(
      'div',
      { className: 'filter number-filter' },
      _react2['default'].createElement(
        'select',
        { ref: 'numberFilterComparator',
          className: 'number-filter-comparator form-control',
          onChange: this.onChangeComparator,
          defaultValue: this.props.defaultValue ? this.props.defaultValue.comparator : '' },
        this.getComparatorOptions()
      ),
      this.props.options ? _react2['default'].createElement(
        'select',
        { ref: 'numberFilter',
          className: selectClass,
          onChange: this.onChangeNumberSet,
          defaultValue: this.props.defaultValue ? this.props.defaultValue.number : '' },
        this.getNumberOptions()
      ) : _react2['default'].createElement('input', { ref: 'numberFilter',
        type: 'number',
        className: 'number-filter-input form-control',
        placeholder: this.props.placeholder || 'Enter ' + this.props.columnName + '...',
        onChange: this.onChangeNumber,
        defaultValue: this.props.defaultValue ? this.props.defaultValue.number : '' })
    );
  };

  return NumberFilter;
})(_react.Component);

NumberFilter.propTypes = {
  filterHandler: _react.PropTypes.func.isRequired,
  options: _react.PropTypes.arrayOf(_react.PropTypes.number),
  defaultValue: _react.PropTypes.shape({
    number: _react.PropTypes.number,
    comparator: _react.PropTypes.oneOf(legalComparators)
  }),
  delay: _react.PropTypes.number,
  /* eslint consistent-return: 0 */
  numberComparators: function numberComparators(props, propName) {
    if (!props[propName]) {
      return;
    }
    for (var i = 0; i < props[propName].length; i++) {
      var comparatorIsValid = false;
      for (var j = 0; j < legalComparators.length; j++) {
        if (legalComparators[j] === props[propName][i]) {
          comparatorIsValid = true;
          break;
        }
      }
      if (!comparatorIsValid) {
        return new Error('Number comparator provided is not supported.\n          Use only ' + legalComparators);
      }
    }
  },
  placeholder: _react.PropTypes.string,
  columnName: _react.PropTypes.string
};

NumberFilter.defaultProps = {
  delay: _Const2['default'].FILTER_DELAY
};

exports['default'] = NumberFilter;
module.exports = exports['default'];