'use strict';

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _PageButtonJs = require('./PageButton.js');

var _PageButtonJs2 = _interopRequireDefault(_PageButtonJs);

var _Const = require('../Const');

var _Const2 = _interopRequireDefault(_Const);

var PaginationList = (function (_Component) {
  _inherits(PaginationList, _Component);

  function PaginationList() {
    var _this = this;

    _classCallCheck(this, PaginationList);

    _Component.apply(this, arguments);

    this.changePage = function (page) {
      var _props = _this.props;
      var prePage = _props.prePage;
      var currPage = _props.currPage;
      var nextPage = _props.nextPage;
      var lastPage = _props.lastPage;
      var firstPage = _props.firstPage;
      var sizePerPage = _props.sizePerPage;

      if (page === prePage) {
        page = currPage - 1 < 1 ? 1 : currPage - 1;
      } else if (page === nextPage) {
        page = currPage + 1 > _this.totalPages ? _this.totalPages : currPage + 1;
      } else if (page === lastPage) {
        page = _this.totalPages;
      } else if (page === firstPage) {
        page = 1;
      } else {
        page = parseInt(page, 10);
      }

      if (page !== currPage) {
        _this.props.changePage(page, sizePerPage);
      }
    };

    this.changeSizePerPage = function (e) {
      e.preventDefault();

      var selectSize = parseInt(e.currentTarget.text, 10);
      var currPage = _this.props.currPage;

      if (selectSize !== _this.props.sizePerPage) {
        _this.totalPages = Math.ceil(_this.props.dataSize / selectSize);
        if (currPage > _this.totalPages) currPage = _this.totalPages;

        _this.props.changePage(currPage, selectSize);
        if (_this.props.onSizePerPageList) {
          _this.props.onSizePerPageList(selectSize);
        }
      }
    };
  }

  PaginationList.prototype.render = function render() {
    var _this2 = this;

    var _props2 = this.props;
    var dataSize = _props2.dataSize;
    var sizePerPage = _props2.sizePerPage;
    var sizePerPageList = _props2.sizePerPageList;

    this.totalPages = Math.ceil(dataSize / sizePerPage);
    var pageBtns = this.makePage();
    var pageListStyle = {
      float: 'right',
      // override the margin-top defined in .pagination class in bootstrap.
      marginTop: '0px'
    };

    var sizePerPageOptions = sizePerPageList.map(function (_sizePerPage) {
      return _react2['default'].createElement(
        'li',
        { key: _sizePerPage, role: 'presentation' },
        _react2['default'].createElement(
          'a',
          { role: 'menuitem',
            tabIndex: '-1', href: '#',
            onClick: _this2.changeSizePerPage },
          _sizePerPage
        )
      );
    });

    return _react2['default'].createElement(
      'div',
      { className: 'row', style: { marginTop: 15 } },
      sizePerPageList.length > 1 ? _react2['default'].createElement(
        'div',
        null,
        _react2['default'].createElement(
          'div',
          { className: 'col-md-6' },
          _react2['default'].createElement(
            'div',
            { className: 'dropdown' },
            _react2['default'].createElement(
              'button',
              { className: 'btn btn-default dropdown-toggle',
                type: 'button', id: 'pageDropDown', 'data-toggle': 'dropdown',
                'aria-expanded': 'true' },
              sizePerPage,
              _react2['default'].createElement(
                'span',
                null,
                ' ',
                _react2['default'].createElement('span', { className: 'caret' })
              )
            ),
            _react2['default'].createElement(
              'ul',
              { className: 'dropdown-menu', role: 'menu', 'aria-labelledby': 'pageDropDown' },
              sizePerPageOptions
            )
          )
        ),
        _react2['default'].createElement(
          'div',
          { className: 'col-md-6' },
          _react2['default'].createElement(
            'ul',
            { className: 'pagination', style: pageListStyle },
            pageBtns
          )
        )
      ) : _react2['default'].createElement(
        'div',
        { className: 'col-md-12' },
        _react2['default'].createElement(
          'ul',
          { className: 'pagination', style: pageListStyle },
          pageBtns
        )
      )
    );
  };

  PaginationList.prototype.makePage = function makePage() {
    var pages = this.getPages();
    return pages.map(function (page) {
      var isActive = page === this.props.currPage;
      var disabled = false;
      var hidden = false;
      if (this.props.currPage === 1 && (page === this.props.firstPage || page === this.props.prePage)) {
        disabled = true;
        hidden = true;
      }
      if (this.props.currPage === this.totalPages && (page === this.props.nextPage || page === this.props.lastPage)) {
        disabled = true;
        hidden = true;
      }
      return _react2['default'].createElement(
        _PageButtonJs2['default'],
        { key: page,
          changePage: this.changePage,
          active: isActive,
          disable: disabled,
          hidden: hidden },
        page
      );
    }, this);
  };

  PaginationList.prototype.getPages = function getPages() {
    var pages = undefined;
    var startPage = 1;
    var endPage = this.totalPages;

    startPage = Math.max(this.props.currPage - Math.floor(this.props.paginationSize / 2), 1);
    endPage = startPage + this.props.paginationSize - 1;

    if (endPage > this.totalPages) {
      endPage = this.totalPages;
      startPage = endPage - this.props.paginationSize + 1;
    }

    if (startPage !== 1 && this.totalPages > this.props.paginationSize) {
      pages = [this.props.firstPage, this.props.prePage];
    } else if (this.totalPages > 1) {
      pages = [this.props.prePage];
    } else {
      pages = [];
    }

    for (var i = startPage; i <= endPage; i++) {
      if (i > 0) pages.push(i);
    }

    if (endPage !== this.totalPages) {
      pages.push(this.props.nextPage);
      pages.push(this.props.lastPage);
    } else if (this.totalPages > 1) {
      pages.push(this.props.nextPage);
    }
    return pages;
  };

  return PaginationList;
})(_react.Component);

PaginationList.propTypes = {
  currPage: _react.PropTypes.number,
  sizePerPage: _react.PropTypes.number,
  dataSize: _react.PropTypes.number,
  changePage: _react.PropTypes.func,
  sizePerPageList: _react.PropTypes.array,
  paginationSize: _react.PropTypes.number,
  remote: _react.PropTypes.bool,
  onSizePerPageList: _react.PropTypes.func,
  prePage: _react.PropTypes.string
};

PaginationList.defaultProps = {
  sizePerPage: _Const2['default'].SIZE_PER_PAGE
};

exports['default'] = PaginationList;
module.exports = exports['default'];