'use strict';

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _objectWithoutProperties = require('babel-runtime/helpers/object-without-properties')['default'];

var _extends = require('babel-runtime/helpers/extends')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _utilsDataStore = require('./utils/dataStore');

var _utilsDataStore2 = _interopRequireDefault(_utilsDataStore);

var _DataTablePagination = require('./DataTablePagination');

var _DataTablePagination2 = _interopRequireDefault(_DataTablePagination);

var DataTable = (function (_React$Component) {
	_inherits(DataTable, _React$Component);

	function DataTable(props) {
		_classCallCheck(this, DataTable);

		_React$Component.call(this, props);

		var paginationProps = this.getPaginationProps(props);
		var currPage = _DataTablePagination.getActivePage(paginationProps);

		this.store = new _utilsDataStore2['default'](props.data, {
			pagination: !!(props.pagination && props.pagination.local),
			currPage: currPage,
			pageSize: paginationProps.pageSize
		});
		this.state = {
			data: this.store.get(),
			currPage: currPage
		};
	}

	DataTable.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
		var paginationProps = this.getPaginationProps(nextProps);
		var currPage = _DataTablePagination.getActivePage(paginationProps);

		nextProps.data && this.store.set(nextProps.data);

		this.setState({
			data: this.store.setPage(currPage).get(),
			currPage: currPage
		});
	};

	DataTable.prototype.render = function render() {
		return _react2['default'].createElement(
			'div',
			{ className: 'data-table-wapper' },
			this.renderDataTable(),
			this.props.pagination ? this.renderPagination() : null
		);
	};

	DataTable.prototype.renderDataTable = function renderDataTable() {
		var _props = this.props;
		var striped = _props.striped;
		var bordered = _props.bordered;
		var hover = _props.hover;
		var serialNumber = _props.serialNumber;
		var dataFields = _props.dataFields;
		var data = _props.data;
		var template = _props.template;
		var rowTemplate = _props.rowTemplate;
		var children = _props.children;

		var otherProps = _objectWithoutProperties(_props, ['striped', 'bordered', 'hover', 'serialNumber', 'dataFields', 'data', 'template', 'rowTemplate', 'children']);

		var classes = {
			'data-table': true,
			'table': true,
			'table-striped': striped,
			'table-bordered': bordered,
			'table-hover': hover
		};

		if (!dataFields || !Array.isArray(dataFields) || dataFields.length <= 0) {
			throw 'Error. 属性 {dataFields} 必传，且为数组。';
		}

		return _react2['default'].createElement(
			'table',
			_extends({}, otherProps, {
				ref: 'table',
				className: _classnames2['default'](classes) }),
			this.renderTableHead(),
			this.renderTableBody()
		);
	};

	DataTable.prototype.renderTableHead = function renderTableHead() {
		var _props2 = this.props;
		var serialNumber = _props2.serialNumber;
		var serialNumberHead = _props2.serialNumberHead;
		var dataFields = _props2.dataFields;

		return _react2['default'].createElement(
			'thead',
			null,
			_react2['default'].createElement(
				'tr',
				null,
				this.renderSerialNumberCell(true, serialNumberHead),
				dataFields.map(function (field) {
					return field.idField ? null : _react2['default'].createElement(
						'th',
						{ key: field.name || 'custom-column' },
						field.text
					);
				})
			)
		);
	};

	DataTable.prototype.renderSerialNumberCell = function renderSerialNumberCell(isHead, cellText) {
		return this.props.serialNumber ? isHead ? _react2['default'].createElement(
			'th',
			{ key: 'no', className: 'cell-no' },
			cellText
		) : _react2['default'].createElement(
			'td',
			{ key: 'no', className: 'cell-no' },
			cellText
		) : null;
	};

	DataTable.prototype.renderTableBody = function renderTableBody() {
		var _this = this;

		var _props3 = this.props;
		var serialNumber = _props3.serialNumber;
		var dataFields = _props3.dataFields;
		var template = _props3.template;
		var rowTemplate = _props3.rowTemplate;
		var emptyText = _props3.emptyText;

		var data = this.state.data;

		if (!data || !Array.isArray(data) || data.length <= 0) {
			return this.renderEmptyText();
		}

		if (rowTemplate) {
			return this.renderTableBodyForRowTemplate();
		}

		return _react2['default'].createElement(
			'tbody',
			null,
			data.map(function (item, index) {
				return _react2['default'].createElement(
					'tr',
					{ key: 'row' + index },
					_this.renderSerialNumberCell(false, ++index),
					dataFields.map(function (field) {
						return field.idField ? null : _react2['default'].createElement(
							'td',
							{ key: field.name || 'custom-column' },
							item[field.name] || field.value || ''
						);
					})
				);
			})
		);
	};

	DataTable.prototype.renderEmptyText = function renderEmptyText() {
		var _props4 = this.props;
		var dataFields = _props4.dataFields;
		var serialNumber = _props4.serialNumber;
		var emptyText = _props4.emptyText;

		var cols = dataFields.length;

		serialNumber && ++cols;

		return _react2['default'].createElement(
			'tbody',
			null,
			_react2['default'].createElement(
				'tr',
				null,
				_react2['default'].createElement(
					'td',
					{ colSpan: cols, className: 'empty-text' },
					emptyText
				)
			)
		);
	};

	DataTable.prototype.renderTableBodyForRowTemplate = function renderTableBodyForRowTemplate() {
		var _this2 = this;

		var _props5 = this.props;
		var serialNumber = _props5.serialNumber;
		var dataFields = _props5.dataFields;
		var rowTemplate = _props5.rowTemplate;

		var data = this.state.data;

		//校验模板是否为 <tr><td></td></tr> 格式
		if (rowTemplate.type !== 'tr') {
			throw 'Error. 行模板 {rowTemplate} 必须为<tr><td></td></tr>元素';
		}

		var cells = rowTemplate.props.children;

		//校验模板中，其TD数量是否和dataFields长度一致
		if (!cells || cells.length !== dataFields.length) {
			throw 'Error. 模板的列数必须和属性 {dataFields} 对应上';
		}

		cells.forEach(function (cell) {
			if (cell.type !== 'td') {
				throw 'Error. 行模板 {rowTemplate} 必须为<tr><td></td></tr>元素';
			}
		});

		var renderChildrenNode = function renderChildrenNode(parent, child, data) {
			if (typeof child === 'string') {
				//当children为string时，把数据占位符%%替换成真实数据
				return child.replace(/%\w*%/g, function (key) {
					return data[key.replace(/%/g, '')] || '';
				});
			} else if (_react.isValidElement(child)) {
				//child为element时，递归调用renderChildrenNode，对所有children的%%数据占位符替换成真实数据。然后return clone后的element
				var node = renderChildrenNode(child, child.props.children, data);
				return _react.cloneElement(child, { children: node });
			}

			return child;
		};

		return _react2['default'].createElement(
			'tbody',
			null,
			data.map(function (dataItem, rowIndex) {
				return _react2['default'].createElement(
					'tr',
					{ key: 'row' + rowIndex },
					_this2.renderSerialNumberCell(false, ++rowIndex),
					dataFields.map(function (field, cellIndex) {
						var _cells$cellIndex$props = cells[cellIndex].props;
						var children = _cells$cellIndex$props.children;
						var childrenNode = _cells$cellIndex$props.childrenNode;
						var otherProps = _cells$cellIndex$props.otherProps;

						if (field.idField === true) {
							//idField为ID标识列，这里直接返回null，暂不作其它处理
							return null;
						}

						if (childrenNode && typeof childrenNode !== 'function') {
							throw 'Error. TD的属性 {childrenNode} 必须为function';
						}

						var column = _react2['default'].createElement('td', _extends({}, otherProps, { key: field.name || 'custom-column' }));

						//如果该列传递了childrenNode，直接调用它得到TD的children
						var columnChildren = childrenNode ? childrenNode(dataItem || {}) : renderChildrenNode(column, children, dataItem || {});

						return _react.cloneElement(column, { children: columnChildren });
					})
				);
			})
		);
	};

	DataTable.prototype.renderPagination = function renderPagination() {
		var pagination = this.props.pagination;

		return _react2['default'].createElement(_DataTablePagination2['default'], _extends({}, pagination, {
			activePage: this.state.currPage,
			onChangePage: pagination.local ? this.handleChangePage.bind(this) : pagination.onChangePage }));
	};

	DataTable.prototype.handleChangePage = function handleChangePage(event, selectedEvent) {
		var currPage = selectedEvent.eventKey;

		this.setState({
			data: this.store.setPage(currPage).get(),
			currPage: currPage
		});
	};

	DataTable.prototype.getPaginationProps = function getPaginationProps(props) {
		return (props || this.props).pagination || _DataTablePagination2['default'].defaultProps;
	};

	return DataTable;
})(_react2['default'].Component);

DataTable.propTypes = {
	/**
  * 是否隔行着色
  */
	striped: _react.PropTypes.bool,
	/**
  * 是否显示border
  */
	bordered: _react.PropTypes.bool,
	/**
  * 是否在hover时，改变行背景色
  */
	hover: _react.PropTypes.bool,
	/**
  * 是否添加序号列
  */
	serialNumber: _react.PropTypes.bool,
	/**
  * 序号列头显示的TEXT
  */
	serialNumberHead: _react.PropTypes.string,
	/**
  * 用于解析data数据的field标识
  */
	dataFields: _react.PropTypes.array.isRequired,
	/**
  * 用于生成DataTable的源数据
  */
	data: _react.PropTypes.array,
	/**
  * 用于生成DataTable的模板（所有TBODY的行）
  */
	template: _react.PropTypes.element,
	/**
  * 用于生成DataTable的行模板
  */
	rowTemplate: _react.PropTypes.element,
	/**
  * 没有数据时，显示的提示信息
  */
	emptyText: _react.PropTypes.string,
	/**
  * 是否添加分页
  */
	pagination: _react.PropTypes.oneOfType([_react.PropTypes.bool, _react.PropTypes.object])
};

DataTable.defaultProps = {
	striped: false,
	bordered: true,
	hover: false,
	serialNumber: false,
	serialNumberHead: '序号',
	emptyText: '没有可显示的数据',
	pagination: false
};

exports['default'] = DataTable;
module.exports = exports['default'];